/* Program13_1 
 *
 * USART1 szvegklds DMA segtsgvel.
 * Az USART1 Tx a DMA1 4. csatornra kapcsoldik,
 * lsd a Reference Manual (RM0008 rev. 16) 78. tblzatt!
 *
 * Az USART1 Tx jelt most az alternatv PB6 kivezetshez rendeljk.
 * Hogy lthassuk az USART1 porton kimen adatokat a PC-n,
 * szksgnk lesz egy USB-soros talaktra. Ennek Rx
 * kivezetst kssk az STM32F103C8 krtya PB6 kivezetsre!
 * Br az RB6 lb 5V tolerns, lehetleg 3,3 V-os talaktt hasznljuk!
 *
 * Az alaprtelmezett 72 MHz rajel frekvencit hasznljuk.
 * Az UART1 modult 9600 Baud sebessgre konfigurltuk.
 *
 * Felhasznlt forrs: 
 * - Mazidi et al., STM32 Arm Programming for Embedded Systems
 *   http://www.microdigitaled.com/ARM/STM_ARM_books.html 
 *
 * Tested with Keil MDK-Lite v5.28 and STM32F1 DFP v2.3.0
 */
#include "stm32f10x.h"

void USART1_init(void);
void DMA1_init(void);
void DMA1_ch4_setup(unsigned int src, unsigned int dst, int len);

int done = 1;
char alphabets[] = "abcdefghijklmnopqrstuvwxyz";
char message[80];
int size = sizeof(alphabets);	

int main (void) {
  USART1_init();
  DMA1_init();
  __enable_irq();                            // Megszaktsok globlis engedlyezse

  while (1) {
    for (int i = 0; i < size; i++)           // az zenet elksztse
      message[i] = alphabets[i];
    while (done == 0) {}                     // Vrjunk, ha a done jelzo 0!
    done = 0;                                // Jelz trlse 
    DMA1_ch4_setup((unsigned int)message, (unsigned int)&USART1->DR, size);
  }
}

/*----------------------------------------------------------------------------
  USART1 s a PB6 kivezets inicializlsa
 *----------------------------------------------------------------------------*/
void USART1_init (void) {
  RCC->APB2ENR |= RCC_APB2ENR_USART1EN;      // USART1 engedlyezse
  RCC->APB2ENR |= RCC_APB2ENR_AFIOEN;        // AFIO rajel engedlyezs
  RCC->APB2ENR |= RCC_APB2ENR_IOPBEN;        // GPIOB rajel engedlyezs
  AFIO->MAPR |= AFIO_MAPR_USART1_REMAP;      // Belltja USART1 REMAP bitjt 
  //--- PB6 konfigurlsa USART1 Tx digitlis kimenetknt -----------
  GPIOB->CRL &= ~(GPIO_CRL_MODE6|GPIO_CRL_CNF6); // PB6 CNF s Mode bitek trlse
  GPIOB->CRL |=  GPIO_CRL_MODE6_0 |          // PA2 pushpull alternate kimenet, max. 10 MHz
                 GPIO_CRL_CNF6_1;            // CNF: 10  MODE: 01

  //--- USART1 konfigurlsa: Tx only, 9600  8 N 1 mdba --------------
  USART1->BRR = 7500;                        // 9600 baud @ 72 MHz (APB2!)
  USART1->CR1 = USART_CR1_TE;                // Tx engedlyezs, 8-bit adat
  USART1->CR2 = 0x0000;                      // 1 stop bit
  USART1->SR = ~USART_SR_TC;                 // Trljk a TC jelzbitet
  USART1->CR1 |= USART_CR1_TCIE;             // tvitel vge megszakts engedlyezse
  NVIC_EnableIRQ(USART1_IRQn);               // USART1 megszakts engedlyezse
  USART1->CR1 |= USART_CR1_UE;               // USART1 mkds engedlyezse
}

/*----------------------------------------------------------------------------
   A DMA1 vezrlo 4. csatorna inicializlsa
 *---------------------------------------------------------------------------*/
void DMA1_init(void) {
  RCC->AHBENR |= RCC_AHBENR_DMA1EN;          // DMA vezrl engedlyezse
  DMA1->IFCR |= DMA_IFCR_CGIF4;              // Trli 4. csatorna megszaktsi jelzbitjeit
  NVIC_EnableIRQ(DMA1_Channel4_IRQn);        // DMA 4. csatorna megszakts enegedlyezs
}

/*----------------------------------------------------------------------------
   DMA 4. csatorna --> UART1 Tx adattvitel konfigursa s indtsa 
 *---------------------------------------------------------------------------*/
void DMA1_ch4_setup(unsigned int src, unsigned int dst, int len) {
  DMA1_Channel4->CCR = 0;                    // DMA1 Channel 4 ideiglenes letiltsa
  while (DMA1_Channel4->CCR & 1) {}          // Vr, amg DMA1 Channel 4 aktv
  DMA1->IFCR |= DMA_IFCR_CGIF4;              // Trli 4. csatorna megszaktsi jelzbitjeit
  DMA1_Channel4->CPAR = dst;                 // Destination  address  (cl)
  DMA1_Channel4->CMAR = src;                 // Source address  (forrs)
  DMA1_Channel4->CNDTR = len;                // length (adatblokk hossza)
  DMA1_Channel4->CCR |= 0x0090;              // MINC=1 PINC=0 CIRC=0 DIR=1 (Read from memory)
  DMA1_Channel4->CCR |= DMA_CCR4_TEIE;       // Transfer Error megszakts engedlyezs
  DMA1_Channel4->CCR |= DMA_CCR4_TCIE;       // Transfer Complete megszakts engedlyezs
  DMA1_Channel4->CCR |= DMA_CCR4_EN;         // DMA1 Channel 4 engedlyezse

  USART1->SR &= ~USART_SR_TC;                // USART tvitel vge jelzbit trlse
  USART1->CR3 = USART_CR3_DMAT;              // DMA Transmit engedlyezs
}

/*----------------------------------------------------------------------------
   DMA1 Channel4 megszakts kiszolglsa

   Ez a fggvny kezeli a DMA1 vezrl 4. csatornjtl rkezo megszaktsokat 
   A hibajelz megszakts itt most csak egy helykitlt vgtelen ciklus, 
   melynek helyre a hiba esetn futtatand kdot rhatjuk.  
   Ha a megszakts a DMA tvitel vgt jelzi, a DMA megszaktsokat letiltjuk
   s a megszaktskr jelzbiteket trljk.
 *---------------------------------------------------------------------------*/
void DMA1_Channel4_IRQHandler(void) {
  if (DMA1->ISR & DMA_ISR_TEIF4)             // Ha tviteli hiba trtnt...
    while(1) {}                              // helyettestsk ezt hibakezelo kddal
  DMA1->IFCR |= DMA_IFCR_CGIF4;              // Trli 4. csatorna megszaktsi jelzbitjeit
  DMA1_Channel4->CCR &= ~(DMA_CCR4_TCIE|     // Transfer Complete megszakts letiltsa
                          DMA_CCR4_TEIE);    // Transfer Error megszakts letiltsa
}

/*----------------------------------------------------------------------------
   USART1 megszakts kiszolglsa
   Az USART1 modul Transmit Complete megszaktst hasznljuk az tvitel vgnek
   jelzsre. Ennek bebillensekor lltjuk 1-be a szemaforknt hasznlt done 
   vltozt, amit a foprogram figyel
 *---------------------------------------------------------------------------*/
void USART1_IRQHandler(void) {
    USART1->SR &= ~USART_SR_TC ;             // UART1 TC jelzbitjnek trlse
    done = 1;                                // A done jelz 1-be lltsa
}
